/* Implementation of NSAutoreleasePool class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: NSAutoreleasePool.m,v 1.4 1996/01/23 12:18:27 tiggr Exp $  */

#define NSAUTORELEASEPOOL_DECLARE_PRIVATE_METHODS
#import "tl/support.h"
#import "tl/NSAutoreleasePool.h"
#import "tl/TLPatchedRoots.h"
#import "tl-private.h"
#import <stdlib.h>

#if !HAVE_REFCOUNTING

/* The current autorelease pool.  XXX Should reside in the thread
   dictionary.  */
static NSAutoreleasePool *current_pool;

@implementation NSAutoreleasePool

+(void) addObject: (id) o
{
  if (!current_pool)
    [self warning: "No current pool for autorelease of %#", o];
  else
    [current_pool addObject: o];
} /* +addObject: */

-(void) activate
{
  child_pool = nil;
  current_pool = self;
} /* -activate */

-(void) addObject: (id) o
{
  if (num == cap)
    {
      cap = cap ? cap * 2 : 4;
      objects = xrealloc (objects, cap * sizeof (objects));
    }
  objects[num++] = o;
} /* -addObject: */

-(void) dealloc
{
  unsigned int i;

  /* Make sure all references from garbage collected TLObjects to reference
     counted objects result in a retain.  This must be done before every
     autorelease.  XXX Doing it twice if our CHILD_POOL is not NIL is
     unnecessary.  */
  [TLObject gc];

  /* Make sure autorelease is possible while we're releasing all our
     objects.  Test whether we have a parent pool in case we're the top
     pool (to keep rc-speed from moaning to the unexpecting user).  */
  if (parent_pool)
    [parent_pool activate];

  /* Fix shit caused by an exception.  */
  if (child_pool)
    [child_pool release];

  /* This better not cause an exception.  Otherwise, we're to protect every
     invocation of release, which will be *expensive*.  */
  for (i = 0; i < num; i++)
    [objects[i] release];

  xfree (objects);
  [super dealloc];
} /* -dealloc */

-init
{
  if (current_pool)
    {
      parent_pool = current_pool;
      [parent_pool setChild: self];
    }
  current_pool = self;
  return (self);
} /* -init */

-(void) setChild: pool
{
  if (child_pool)
    [self warning: "setChild: %#, we already had child %#", pool, child_pool];
  child_pool = pool;
} /* -setChild: */

@end

#endif
