/* Implementation of TLInetSocketStream class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLInetSocketStream.m,v 1.6 1996/01/23 12:18:32 tiggr Exp $  */

#define TLFDSTREAM_DECLARE_PRIVATE_METHODS
#define TLSOCKETSTREAM_DECLARE_PRIVATE_METHODS
#define TLINETSOCKETSTREAM_DECLARE_PRIVATE_METHODS
#import "tl/TLInetSocketStream.h"
#import "tl/TLPatchedRoots.h"
#import "tl/TLString.h"
#import <sys/socket.h>
#import <netinet/in.h>
#import <netdb.h>
#import <string.h>
#import "config.h"
#if HAVE_UNISTD_H
#import <unistd.h>
#endif
#if HAVE_STRINGS_H
#import <strings.h>
#endif

@implementation TLInetSocketStream

-(TLSocketStream *) accept
{
  /* It's a pitty we need this struct here...  */
  struct sockaddr_in addr;
  int nfd, addr_len = sizeof (addr);
  id <TLNetworkAddress> naddr;

  bzero ((void *) &addr, sizeof (addr));
  /* The cast to void is since some systems want (struct sockaddr_in *)
     and other desire (struct sockaddr *).  */
  nfd = accept (fd, (void *) &addr, &addr_len);
  if (nfd == -1)
    {
      [self warning: "accept (%#): %s", self, ERRMSG];
      return (nil);
    }

  /* XXX Yes, but I receive a *class* object so it *does* implement this
     method.  */
  naddr = [(id) [self addressClass]
	   addressWithAddress: &addr length: sizeof (addr)];
  if (!naddr)
    return (nil);
  return ([[self class] streamWithFD: nfd address: naddr]);
} /* -accept */

-(id) addressClass
{
  return ([TLInetAddress self]);
} /* -addressClass */

-initWithAddress: (id <TLInetAddress>) addr
{
  const char *s = [[addr protocol] cString];
  struct protoent *ent = getprotobyname ((void *) s);

  address = addr;
  if (!ent)
    {
      [self error: "Unknown protocol: `%s'", s];
      INIT_RETURN_NIL ();
    }

  fd = socket (AF_INET, SOCK_STREAM, ent->p_proto);
  if (fd == -1)
    {
      [self error: "socket (%s): %s", s, ERRMSG];
      INIT_RETURN_NIL ();
    }
  return (self);
} /* -initWithAddress: */

-(id <TLNetworkAddress>) peerAddress
{
  /* It's a pitty we need this struct here...  */
  struct sockaddr_in addr;
  int i, addr_len = sizeof (addr);

  bzero ((void *) &addr, sizeof (addr));
  i = getpeername (fd, (void *) &addr, &addr_len);
  if (i)
    {
      if (ERRNO != ENOTCONN)
	[self warning: "getpeername (%#): %s", self, ERRMSG];
      return (nil);
    }

  return [(id) [self addressClass]
	       addressWithAddress: &addr length: sizeof (addr)];
} /* -peerAddress */

@end
