/* Implementation of TLSyslogStream class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLSyslogStream.m,v 1.4 1996/01/24 18:53:28 tiggr Exp $  */

#define TLSYSLOGSTREAM_DECLARE_PRIVATE_METHODS
#import "tl/support.h"
#import "tl/TLSyslogStream.h"
#import <syslog.h>
#import <string.h>

@implementation TLSyslogStream

+(void) initializeFacility: (int) fac
		     ident: (id <TLString>) ident
		   options: (int) opt
{
  static id <TLString> identity;

  if (identity && ident)
    {
      [(id) identity gcUnlock];
      identity = 0;
      closelog ();
    }
  
  if (ident)
    {
      identity = ident;
      [(id) identity gcLock];
    }

  openlog ([identity cString], opt, fac);
} /* +initializeFacility:ident:options: */

+(id <TLStream, TLOutputStream>) streamWithPriority: (int) prio
{
  return ([[self gcAlloc] initWithPriority: prio]);
} /* +streamWithPriority: */

-(void) gcReference
{
  MARK (prefix);

  [super gcReference];
} /* -gcReference */

-initWithPriority: (int) prio
{
  priority = prio;
  return (self);
} /* -initWithPriority:options:facility:ident: */

-(void) setPrefix: (id) p
{
  ASGN_IVAR (prefix, p);
} /* -setPrefix: */

/******************** TLStream ********************/

-close
{
  if (num)
    [self writeByte: '\n'];
  return (self);
} /* -close */

-(int) fileDescriptor
{
  return (-1);
} /* -fileDescriptor */

-streamp
{
  return (self);
} /* -streamp */

/******************** TLOutputStream ********************/

-flushOutput
{
  return (self);
} /* -flushOutput */

-(int) writeByte: (char) c
{
  return ([self writeBytes: 1 fromBuffer: &c] == 1 ? c : TL_EOF);
} /* -writeByte: */

-(int) writeBytes: (int) n fromBuffer: (const char *) b
{
  int i, start;

  if (num + n > cap)
    {
      cap = num + n;
      buf = xrealloc (buf, cap);
    }

  memcpy (buf + num, b, n);
  num += n;

  /* Output any full lines.  */
  for (start = 0, i = num - n; i < num; i++)
    if (buf[i] == '\n')
      {
	buf[i] = 0;
	if (prefix)
	  syslog (priority, "%s: %s", [formac (nil, @"%@", prefix) cString],
		  buf + start);
	else
	  syslog (priority, "%s", buf + start);
	start = i + 1;
      }

  if (start)
    {
      if (num != start)
	memmove (buf, buf + start, num - start);
      num -= start;
    }

  return (n);
} /* -writeBytes:fromBuffer: */

@end
