/* Test refcounting implementation speed.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: rc-speed.m,v 1.3 1996/01/23 12:19:05 tiggr Exp $  */

#ifdef USE_LIBOBJECTS
#import <objects/AutoreleasePool.h>
#define NSAutoreleasePool AutoreleasePool
static double get_run_time (void);
#else
# ifdef USE_FOUNDATION
# import <stdio.h>
# import <objc/Object.h>
# import <foundation/NSAutoreleasePool.h>
static double get_run_time (void);
# else
# import <tl/tl.h>
# endif
#endif
#import <stdlib.h>

int
main (int argc, char **argv)
{
  int i, n = 0;
  id *q;
  double start, end;
  id pool;

  if (argc == 2)
    n = atoi (argv[1]);
  else if (argc == 1)
    {
      n = 100000;
      fprintf (stderr, "using default n=%d\n", n);
    }
  if (argc > 2 || !n)
    {
      fprintf (stderr, "usage: %s [n]\n", *argv);
      exit (2);
    }

#if !defined (USE_LIBOBJECTS) && !defined (USE_FOUNDATION)
  tl_init ();
#endif

  [[Object alloc] init];

  q = calloc (n, sizeof (*q));

  start = get_run_time ();
  for (i = 0; i < n; i++)
    ;
  end = get_run_time ();
  printf ("%d empty loop: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    q[i] = [[Object alloc] init];
  end = get_run_time ();
  printf ("%d [[Object alloc] init]: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] retain];
  end = get_run_time ();
  printf ("%d first retain: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] retain];
  end = get_run_time ();
  printf ("%d retain: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] retain];
  end = get_run_time ();
  printf ("%d retain: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] retain];
  end = get_run_time ();
  printf ("%d retain: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] retain];
  end = get_run_time ();
  printf ("%d retain: %f\n", n, end - start);

  pool = [[NSAutoreleasePool alloc] init];

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] autorelease];
  end = get_run_time ();
  printf ("%d autorelease: %f\n", n, end - start);

  start = get_run_time ();
  [pool release];
  end = get_run_time ();
  printf ("[current_autoreleasepool release]: %f\n", end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] release];
  end = get_run_time ();
  printf ("%d release: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] release];
  end = get_run_time ();
  printf ("%d release: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] release];
  end = get_run_time ();
  printf ("%d release: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] release];
  end = get_run_time ();
  printf ("%d release: %f\n", n, end - start);

  start = get_run_time ();
  for (i = 0; i < n; i++)
    [q[i] release];
  end = get_run_time ();
  printf ("%d last release: %f\n", n, end - start);

  return (0);
}

#if defined (USE_LIBOBJECTS) || defined (USE_FOUNDATION)

/******************** runtime stuff (from gcc) ********************/

#ifdef hpux
#define USG
#endif
#ifdef USG
#undef FLOAT
#include <sys/param.h>
/* This is for hpux.  It is a real screw.  They should change hpux.  */
#undef FLOAT
#include <sys/times.h>
#include <time.h>   /* Correct for hpux at least.  Is it good on other USG?  */
#undef FFS  /* Some systems define this in param.h.  */
#else
#ifndef VMS
#include <sys/time.h>
#include <sys/resource.h>
#endif
#endif

static double
get_run_time (void)
{
#ifdef USG
  struct tms tms;
#else
#ifndef VMS
  struct rusage rusage;
#else /* VMS */
  struct
    {
      int proc_user_time;
      int proc_system_time;
      int child_user_time;
      int child_system_time;
    } vms_times;
#endif
#endif

#ifdef USG
  times (&tms);
  return ((double) (tms.tms_utime + tms.tms_stime) / HZ);
#else
#ifndef VMS
  getrusage (0, &rusage);
  return ((double) rusage.ru_utime.tv_sec + rusage.ru_utime.tv_usec / 1e6
	  + (double) rusage.ru_stime.tv_sec + rusage.ru_stime.tv_usec / 1e6);
#else /* VMS */
  times (&vms_times);
  return (vms_times.proc_user_time + vms_times.proc_system_time) / 100.0;
#endif
#endif
} /* get_run_time */

#endif
