/* Interface to TLSocketStream classes.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLSocketStream.h,v 1.4 1996/01/23 12:19:23 tiggr Exp $  */

#import "TLFDStream.h"

@protocol TLNetworkAddress

+(id <TLNetworkAddress>) addressWithAddress: (void *) addr length: (int) len;

-(void *) address;

-(int) addressSize;

-(id) socketClass;

@end

@class TLSocketStream;

@protocol TLNetworkStream

/* Both the address and the returned value are (should be) specific to the
   receiving class.  Thus, an TLInetSocketStream will expect a TLInetAddress
   and will return an instance of itself.  */
+(TLSocketStream *) streamWithAddress: (id <TLNetworkAddress>) addr;

/* THis is called internally in response to an accept.  */
+(TLSocketStream *) streamWithFD: (int) d address: (id <TLNetworkAddress>) addr;

-(id) initWithAddress: (id <TLNetworkAddress>) addr;

-(TLSocketStream *) accept;

-(id <TLNetworkAddress>) address;

/* The preferred class from which to instantiate new addresses (for an
   accept).  */
-(id <TLNetworkAddress>) addressClass;

/* For a connected socket, return the address of the peer.  */
-(id <TLNetworkAddress>) peerAddress;

-bind;

-connect;

-listen;

@end

@interface TLSocketStream: TLFDStream <TLNetworkStream>
{
  id <TLNetworkAddress> address;
}

#ifdef TLSOCKETSTREAM_DECLARE_PRIVATE_METHODS

+(TLSocketStream *) streamWithFD: (int) d address: (id <TLNetworkAddress>) addr;

-(void) setAddress: (id <TLNetworkAddress>) address;

#endif

@end
