/* Interface to TLTelnetStream class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLTelnetStream.h,v 1.4 1996/01/23 12:19:25 tiggr Exp $  */

#ifdef TLTELNETSTREAM_DECLARE_PRIVATE_METHODS
#define TLSTREAMSTREAM_DECLARE_PRIVATE_METHODS
#endif

#import "TLStreamStream.h"

enum tl_telnet_options
{
  TELOPT_TRANSMIT_BINARY = 0,		/* rfc856 */
  TELOPT_ECHO = 1,			/* rfc857 */
  TELOPT_SUPPRESS_GO_AHEAD = 3,		/* rfc858 */
  TELOPT_STATUS = 5,			/* rfx859 */
  TELOPT_TIMEING_MARK = 6,		/* rfc860 */
  TELOPT_EXTEND_ASCII = 17,		/* rfc698 */
  TELOPT_DET = 20,			/* rfc1043 */
  TELOPT_TERMINAL_TYPE = 24,		/* rfc1091 */
  TELOPT_END_OF_RECORD = 25,		/* rfc885 */
  TELOPT_TUID = 26,			/* rfc927 */
  TELOPT_OUTMRK = 27,			/* rfc933 */
  TELOPT_TTYLOC = 28,			/* rfc946 */
  TELOPT_3270_REGIME = 29,		/* rfc1041 */
  TELOPT_X_3_PAD = 30,			/* rfc1053 */
  TELOPT_NAWS = 31,			/* rfc1073 */
  TELOPT_TERMINAL_SPEED = 32,		/* rfc1079 */
  TELOPT_TOGGLE_FLOW_CONTROL = 33,	/* rfc1372 */
  TELOPT_LINEMODE = 34,			/* rfc1184 */
  TELOPT_XDISPLOC = 35,			/* rfc1096 */
  TELOPT_ENVIRON = 36,			/* rfc1408 */
  TELOPT_AUTHENTICATION = 37,		/* rfc1416 */
  TELOPT_NEW_ENVIRON = 39,		/* rfc1572 */
  TELOPT_EXOPL = 255,			/* rfc861 */
  TELOPT_SUBLIMINAL_MESSAGE = 257,	/* rfc1097 */
};

enum tl_telnet_character_state
{
  TELCS_DATA,
  TELCS_NUL = 0,
  TELCS_NL = 10,
  TELCS_CR = 13,
  TELCS_SE = 240,
  TELCS_NOP,
  TELCS_DM,
  TELCS_BRK,
  TELCS_IP,
  TELCS_AO,
  TELCS_AYT,
  TELCS_EX,
  TELCS_EL,
  TELCS_GA,
  TELCS_SB,
  TELCS_WILL,
  TELCS_WONT,
  TELCS_DO,
  TELCS_DONT,
  TELCS_IAC
};

/* Option negotiation FSM states as in rfc1143.  */
enum tl_telnet_option_state
{
  TELOS_NO,
  TELOS_WANTNO_EMPTY,
  TELOS_WANTNO_OPPOSITE,
  TELOS_WANTYES_EMPTY,
  TELOS_WANTYES_OPPOSITE,
  TELOS_YES,
};

/* The number of options we handle.  Some don't even exist yet.  */
#define TLTELNET_NUM_OPTIONS  40

/* This stream implements the TELNET protocol, as in rfc854.  */
@interface TLTelnetStream: TLStreamStream <TLStream, TLInputOutputStream>
{
  /* The state of the incoming stream.  */
  enum tl_telnet_character_state state;

  /* Option negotiation state machine.  Index 0 is him, 1 is us.  Pitty that
     we can't declare an array of 8-bit bitfields of an enum type.  */
  unsigned char osm[2][TLTELNET_NUM_OPTIONS];

  /* The delegate deciding in options.  */
  id delegate;

  /* Iff !0, we're getting a suboption (finished by an SE command).  */
  unsigned int subnegotiation: 1;
}

#ifdef TLTELNETSTREAM_DECLARE_PRIVATE_METHODS

#endif

@end

/* Make sure the delegate methods are known.  */
@interface TLTelnetStream (DelegateMethods)

-(id) telnetStream: (TLTelnetStream *) s
	optionWill: (enum tl_telnet_options) o;

-(id) telnetStream: (TLTelnetStream *) s
	optionWont: (enum tl_telnet_options) o;

-(id) telnetStream: (TLTelnetStream *) s
	  optionDo: (enum tl_telnet_options) o;

-(id) telnetStream: (TLTelnetStream *) s
	optionDont: (enum tl_telnet_options) o;

@end
